package speiger.src.coreengine.rendering.gui.screen;

import java.util.List;

import speiger.src.collections.ints.sets.IntOpenHashSet;
import speiger.src.collections.ints.sets.IntSet;
import speiger.src.collections.objects.lists.ObjectArrayList;
import speiger.src.collections.objects.utils.ObjectIterables;
import speiger.src.collections.objects.utils.ObjectIterators;
import speiger.src.coreengine.rendering.gui.components.base.GuiComponent;
import speiger.src.coreengine.rendering.gui.components.base.IComponentScreen;
import speiger.src.coreengine.rendering.gui.components.base.IInteractable;
import speiger.src.coreengine.rendering.gui.components.base.IInteractableContainer;
import speiger.src.coreengine.rendering.guiOld.helper.box.IGuiBox;

public class GuiContainer implements IComponentScreen, IInteractableContainer {
	LayeredContainer container = new LayeredContainer();
	IntSet activeButtons = new IntOpenHashSet();
	IInteractable focused;
	IGuiBox box = IGuiBox.of(0F, 0F, 0F, 0F);
	long tick;
	
	@Override
	public IGuiBox getBox() { return box; }
	@Override
	public long clock() { return tick; }
	@Override
	public boolean isFocused() { return focused != null; }
	@Override
	public void setFocused(boolean value) { if(!value) setFocusedChild(null); }
	@Override
	public List<? extends IInteractable> getChildren() { return container.interactables(); }
	@Override
	public void setFocusedChild(IInteractable child) {
		if(focused != null) focused.setFocused(false);
		this.focused = child;
		if(focused != null) focused.setFocused(true);
	}
	@Override
	public IInteractable getFocusedChild() { return focused; }
	@Override
	public IntSet getActiveButtons() { return activeButtons; }
	
	public static class LayeredContainer {
		List<List<GuiComponent>> components = new ObjectArrayList<>();
		List<List<IInteractable>> interactables = new ObjectArrayList<>();
		int layer = -1;
		
		public LayeredContainer() { pushLayer(); }
		
		public void pushLayer() {
			layer++;
			components.add(new ObjectArrayList<>());
			interactables.add(new ObjectArrayList<>());
		}
		
		public void popLayer() {
			if(layer == 0) return;
			layer--;
			components.removeLast();
			interactables.removeLast();
		}
		
		public void add(GuiComponent component) {
			children().add(component);
			interactables().add(component.interactContainer());
		}
		
		public List<GuiComponent> children() {
			return components.get(layer);
		}
		
		public List<IInteractable> interactables() {
			return interactables.get(layer);
		}
	}
}
